/* Classic Ladder Project */
/* Copyright (C) 2001-2010 Marc Le Douarain */
/* http://www.multimania.com/mavati/classicladder */
/* http://www.sourceforge.net/projects/classicladder */
/* February 2001 */
/* ------------------------------ */
/* The main for Linux Application */
/* ------------------------------ */
/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifdef MODULE
#include <linux/string.h>
#else
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#endif

#include "classicladder.h"
#include "global.h"
#include "files_project.h"
#include "calc.h"
#include "vars_access.h"
#include "manager.h"
#include "calc_sequential.h"
#include "files_sequential.h"
//#include "config.h"
#include "hardware.h"
#include "socket_server.h"
#include "socket_modbus_master.h"
#include "vars_system.h"

#if !defined( MODULE )
#include <gtk/gtk.h>
#include "classicladder_gtk.h"
#include "manager_gtk.h"
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <signal.h>
#include <getopt.h>
#ifdef __WIN32__
#include <windows.h>
#else
#include <pthread.h>
#endif
#ifdef __XENO__
#include <sys/mman.h>
#endif
#endif

#ifdef GTK_INTERFACE
#include <gtk/gtk.h>
#include "menu_and_toolbar_gtk.h"
#endif

#ifdef MAT_CONNECTION
#include "../../lib/plc.h"
#endif

int cl_remote;
int nogui = 0;
int ModbusServerPort = 9502; // Standard "502" requires root privileges...

#if !defined( MODULE )

#ifdef __WIN32__
HANDLE ThreadHandleCyclicRefresh = NULL;
DWORD ThreadIdCyclicRefresh;
#else
pthread_t thread_cyclic_refresh;
#endif
int CyclicThreadRunning = 0;

#ifdef GTK_INTERFACE
char TheProject[400] = "projects_examples/example.clp";
#else
char TheProject[400] = "projects_examples/parallel_port_direct.clp";
#endif


void ClassicLadderEndOfAppli( void )
{
	CyclicThreadRunning = 0;
	StopRunIfRunning( );
#ifdef MODBUS_IO_MASTER
	CloseSocketModbusMaster( );
#endif
	CloseSocketServer( );
	ClassicLadder_FreeAll( TRUE/*CleanAndRemoveTmpDir*/ );
}

void HandlerSignalInterrupt( int signal_id )
{
	debug_printf("End of application asked\n");
	ClassicLadderEndOfAppli( );
	exit( 0 );
}


void display_help (void)
{
	printf("Usage: classicladder [OPTIONS] [PATH]\n"
			"Start classicladder PLC with an optional project path e.g. myplc\n"
			"\n"
			"           --help     	        display this help and exit\n"
			"           --version  	        output version information and exit\n"
			"           --nogui             do not create the GUI\n"
			"-p port    --modbus_port=port   port to use for modbus server\n"
//			"-c file    --config=file        read PLC configuration from file\n"
			);
	exit(0);
}

void display_version (void)
{
	printf( CL_PRODUCT_NAME " v" CL_RELEASE_VER_STRING "\n" CL_RELEASE_DATE_STRING "\n"
	       "\n"
               "Copyright (C) " CL_RELEASE_COPYRIGHT_YEARS " Marc Le Douarain\nmarc . le - douarain /At\\ laposte \\DoT/ net\n"
	       "\n"
	       "ClassicLadder comes with NO WARRANTY\n"
	       "to the extent permitted by law.\n"
	       "\n"
	       "You may redistribute copies of ClassicLadder\n"
	       "under the terms of the GNU Lesser General Public Licence.\n"
	       "See the file `lesserGPL.txt' for more information.\n");
	exit(0);
}

void process_options (int argc, char *argv[])
{
	int error = 0;

	for (;;)
	{
		int option_index = 0;
		static const char *short_options = "c:";
		static const struct option long_options[] = {
			{"help", no_argument, 0, 0},
			{"version", no_argument, 0, 0},
			{"nogui", no_argument, 0, 'n'},
//			{"config", required_argument, 0, 'c'},
			{"modbus_port", required_argument, 0, 'p'},
			{0, 0, 0, 0},
		};

		int c = getopt_long(argc, argv, short_options,
				    long_options, &option_index);
		if (c == EOF)
			break;

		switch (c)
		{
			case 0:
				switch (option_index) {
				case 0:
					display_help();
					break;
				case 1:
					display_version();
					break;
				}
				break;
			case 'n':
				nogui = 1;
				break;
/*			case 'c':
	#ifndef RT_SUPPORT
				read_config (optarg);
	#else
				printf("Config file is used by the RT module in RTLinux version !!!\n");
	#endif
				break;*/
			case 'p':
				ModbusServerPort = atoi( optarg );
				break;
			case '?':
				error = 1;
				break;
		}
	}

	if (error)
		display_help ();

	if ((argc - optind) != 0)
		VerifyDirectorySelected( TheProject, argv[optind] );
}

void DoPauseMilliSecs( int MilliSecsTime )
{
#ifdef __WIN32__
	Sleep( MilliSecsTime );
#else
	struct timespec time;
	int NbrSecs =0;
	int NbrNanos = MilliSecsTime*1000000;
	if ( MilliSecsTime>=1000 )
	{
		NbrSecs = MilliSecsTime/1000;
		NbrNanos = (MilliSecsTime%1000)*1000000;
	}
	time.tv_sec = NbrSecs;
	time.tv_nsec = NbrNanos;
	nanosleep( &time, NULL );
	//usleep( Time*1000 );
#endif
}

void DoFlipFlopRunStop( void )
{
	if (InfosGene->LadderState==STATE_RUN)
	{
		InfosGene->LadderState = STATE_STOP;
#ifdef GTK_INTERFACE
//		gtk_label_set_text(GTK_LABEL(GTK_BIN(ButtonRunStop)->child),"Run");
		MessageInStatusBar("Stopped program - press run button to continue.");
//		SetToggleMenuForRunStop( TRUE );
		SetMenuStateForRunStopSwitch( FALSE );
#endif
	}
	else
	{
		InfosGene->LadderState = STATE_RUN;
#ifdef GTK_INTERFACE
//		gtk_label_set_text(GTK_LABEL(GTK_BIN(ButtonRunStop)->child),"Stop");
		MessageInStatusBar("Started program - press stop to pause."); 
//		SetToggleMenuForRunStop( FALSE );
		SetMenuStateForRunStopSwitch( TRUE );
#endif
	}
}

void StopRunIfRunning( void )
{
	if (InfosGene->LadderState==STATE_RUN)
	{
		debug_printf("Stopping...");
		InfosGene->LadderStoppedToRunBack = TRUE;
		InfosGene->LadderState = STATE_STOP;
		while( InfosGene->UnderCalculationPleaseWait==TRUE )
		{
			DoPauseMilliSecs( 100 );
		}
		debug_printf("done.\n");
	}
}
void RunBackIfStopped( void )
{
	if ( InfosGene->LadderStoppedToRunBack )
	{
		debug_printf("Start running!\n");
		InfosGene->LadderState = STATE_RUN;
		InfosGene->LadderStoppedToRunBack = FALSE;
	}
}

void CyclicCalcRefresh( void )
{
#ifdef __XENO__
	struct timespec start, period, now;
	unsigned long overruns;
	int err = 0;
	pthread_set_name_np( pthread_self(), __FUNCTION__ );
//TODO: support PeriodicRefreshMilliSecs instead of PERIODIC_REFRESH_MS_DEF...
	clock_gettime( CLOCK_REALTIME, &start );
	start.tv_sec += 1;	/* Start in one second from now. */
	period.tv_sec = 0;
	period.tv_nsec = PERIODIC_REFRESH_MS_DEF*1000000;
	if ( pthread_make_periodic_np(pthread_self(), &start, &period)!=0 )
	{
		printf("Xenomai make_periodic failed: err %d\n", err);
		exit(EXIT_FAILURE);
	}
#endif
//	while( CyclicThreadRunning )
	do
	{
#ifdef __XENO__
		err = pthread_wait_np(&overruns);
		if (err || overruns) {
			printf( "Xenomai wait_period failed: err %d, overruns: %lu\n", err, overruns);
//			exit(EXIT_FAILURE);
		}
#else
		DoPauseMilliSecs( InfosGene->GeneralParams.PeriodicRefreshMilliSecs/*PERIODIC_REFRESH_MS_DEF*/ );
#endif

		if ( CyclicThreadRunning )
		{
			if (InfosGene->LadderState==STATE_RUN)
			{
				// called here, because ioperm seems requiring to be defined in this thread using them.
				if ( InfosGene->AskToConfHard )
				{
					ConfigHardware( );
					InfosGene->AskToConfHard = FALSE;
				}

	#ifdef __XENO__
				clock_gettime( CLOCK_REALTIME, &start );
	#endif
				InfosGene->UnderCalculationPleaseWait = TRUE;
				ReadPhysicalInputs();
				UpdateSystemVars( );
				ClassicLadder_RefreshAllSections();
				WritePhysicalOutputs();
				InfosGene->UnderCalculationPleaseWait = FALSE;

	#ifdef __XENO__
				clock_gettime( CLOCK_REALTIME, &now );
				if ( now.tv_sec==start.tv_sec )
					InfosGene->DurationOfLastScan = now.tv_nsec-start.tv_nsec;
	#endif
			}
		}
	}
	while( CyclicThreadRunning );
#ifndef __WIN32__
	pthread_exit(NULL);
#endif
}

int InitCyclicThread( void )
{
	int InitOk = 0;
#ifndef __WIN32__
	int Error = 0;
#endif
	CyclicThreadRunning = 1;
#ifdef __WIN32__
	ThreadHandleCyclicRefresh = CreateThread( NULL/*no security attributes*/, 16*1024L/*default stack size*/,                                                   
			(LPTHREAD_START_ROUTINE)CyclicCalcRefresh/* thread function*/, 
			NULL/*argument to thread function*/,                
			THREAD_QUERY_INFORMATION/*use default creation flags*/,                           
			&ThreadIdCyclicRefresh/* returns the thread identifier*/);                
	if ( ThreadHandleCyclicRefresh==NULL )
#else
	pthread_attr_t ThreadAttributes;
	pthread_attr_init(&ThreadAttributes);
	pthread_attr_setdetachstate(&ThreadAttributes, PTHREAD_CREATE_DETACHED /*PTHREAD_CREATE_JOINABLE*/);
#ifdef __XENO__
	pthread_attr_setschedpolicy(&ThreadAttributes, SCHED_FIFO);
	struct sched_param paramA = { .sched_priority = 98 };
	pthread_attr_setschedparam(&ThreadAttributes, &paramA);
#endif
	pthread_attr_setstacksize(&ThreadAttributes, 32*1024);
	Error = pthread_create( &thread_cyclic_refresh, &ThreadAttributes, (void *(*)(void *))CyclicCalcRefresh, (void *)NULL );
	if (Error) // pthread_create returns 0 on success
#endif
		debug_printf("Failed to create cyclic calc thread...\n");
	else
		InitOk = 1;
	return InitOk;
}

int main( int   argc, char *argv[] )
{
#ifdef __XENO__
	mlockall(MCL_CURRENT | MCL_FUTURE);
#endif

	InitHardware( );
#ifdef MODBUS_IO_MASTER
	InitModbusMasterBeforeReadConf( );
#endif
	process_options (argc, argv);

	// here we get the sizes defined in the project...
	// (before doing the allocations !)
	LoadGeneralParamsOnlyFromProject( TheProject );

	if (ClassicLadder_AllocAll())
	{
		// now InfosGene allocated...
		strcpy( InfosGene->CurrentProjectFileName, TheProject );

		InitSocketServer( 0/*UseUdpMode*/, ModbusServerPort/*PortNbr*/ );
#ifdef MODBUS_IO_MASTER
		InitSocketModbusMaster( );
#endif
		signal( SIGINT /*SIGTERM*/, HandlerSignalInterrupt );

// cyclic thread for rt is in the kernel (RTLinux/RTAI)
#ifndef RT_SUPPORT
		if ( InitCyclicThread( ) )
#endif
		{
			char WaitHere = TRUE;
			char ProjectLoadedOk;

#ifdef GTK_INTERFACE
			if (nogui==0 )
				InitGtkWindows( argc, argv );
#endif
	
			//TODO: perhaps pass here TheProject as parameter, and is copied internally
			//into InfosGene->CurrentProjectFileName in the function LoadProjectFiles() ?!
			//lot of works to do in the gtk part for LoadProjectFiles & SaveProjectFiles...
			ProjectLoadedOk = LoadProjectFiles( InfosGene->CurrentProjectFileName );
#ifdef GTK_INTERFACE
			if (nogui==0 )
			{
				UpdateAllGtkWindows( );
				UpdateWindowTitleWithProjectName( );
				MessageInStatusBar( ProjectLoadedOk?"Project loaded and running":"Project failed to load...");
			}
#endif

#ifndef RT_SUPPORT
			OpenHardware( 0 );
//			ConfigHardware( );
			InfosGene->AskToConfHard = TRUE;
#endif

			InfosGene->LadderState = STATE_RUN;

#ifdef GTK_INTERFACE
			//ProblemWithPrint		gdk_threads_enter( );
			if (nogui==0 )
			{
				gtk_main();
				WaitHere = FALSE;
			}
			//ProblemWithPrint		gdk_threads_leave( );
#endif
			if ( WaitHere )
			{
				debug_printf("Press CTRL-C to end...!\n");
				for( ;; )
					DoPauseMilliSecs( 1000 );
			}

#ifdef __WIN32__
			if ( ThreadHandleCyclicRefresh )
				TerminateThread( ThreadHandleCyclicRefresh, 0);
#endif	
		}
	}
	else
	{
		ClassicLadder_FreeAll( TRUE/*CleanAndRemoveTmpDir*/ );
	}

	return 0;
}
#endif
