/* Classic Ladder Project */
/* Copyright (C) 2001 Marc Le Douarain */
/* mavati@club-internet.fr */
/* http://www.multimania.com/mavati/classicladder */
/* January 2002 */
/* ------------------ */
/* Hardware Interface */
/* ------------------ */
/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "classicladder.h"
#include "global.h"
#include "vars_access.h"
#include "hardware.h"

#ifdef COMEDI_SUPPORT
#ifdef __RTL__
#include <rtl.h>
#include <linux/comedilib.h>
#else
#include <comedilib.h>
#endif
#endif

#if !defined( MODULE )
#include <stdio.h>
#include <unistd.h>
#ifdef MAKE_IO_ACCESS
#include <sys/io.h>
#endif
#else
#include <asm/io.h>
#endif
#ifdef RTAI
#include "rtai.h"
#endif

#define NBR_COMEDI_DEVICES 4

#ifdef COMEDI_SUPPORT
comedi_t * ComediDev[ NBR_COMEDI_DEVICES ];
#endif

void ReadPhysicalInputs( )
{
	int NumConf;
	StrIOConf * pConf;
	unsigned int DatasRead;
	unsigned int Mask;
	int VarNumber;
	for( NumConf=0; NumConf<NBR_INPUTS_CONF; NumConf++ )
	{
		pConf = &InfosGene->InputsConf[ NumConf ];
		/* something mapped ? */
		if ( pConf->FirstClassicLadderIO!=-1 )
		{
			DatasRead = 0;
			/* read the physical inputs */
#ifdef MAKE_IO_ACCESS
			if ( pConf->DeviceType==DEVICE_TYPE_DIRECT_ACCESS )
			{
				DatasRead = inb( pConf->SubDevOrAdr );
			}
#endif
#ifdef COMEDI_SUPPORT
			if ( pConf->DeviceType>=DEVICE_TYPE_COMEDI )
			{
				int ComediNum = pConf->DeviceType-DEVICE_TYPE_COMEDI;
				/* device is opened ? */
				if ( ComediDev[ ComediNum ] )
				{
					/* try to use the optimized way if possible */
					if ( pConf->FirstChannel<=31 && pConf->NbrConsecutivesChannels+pConf->FirstChannel<=32 )
					{
						comedi_dio_bitfield( ComediDev[ ComediNum ],
									pConf->SubDevOrAdr, 0, &DatasRead );
					}
					else
					{
						int ScanEach;
						int BitValue;
						Mask = 1<<pConf->FirstChannel;
						for( ScanEach=pConf->FirstChannel; ScanEach<pConf->FirstChannel+pConf->NbrConsecutivesChannels; ScanEach++ )
						{
							comedi_dio_read( ComediDev[ ComediNum ],
									pConf->SubDevOrAdr, ScanEach, &BitValue );
							if ( BitValue )
								DatasRead = DatasRead|Mask;
							Mask = Mask<<1;
						}
					}
				}
			}
#endif

			if ( pConf->FlagInverted )
				DatasRead = ~DatasRead;

			/* Copy the datas read on the inputs variables */
			Mask = 1<<pConf->FirstChannel;
			for (VarNumber=pConf->FirstClassicLadderIO; VarNumber<pConf->FirstClassicLadderIO+pConf->NbrConsecutivesChannels; VarNumber++)
			{
				WriteVar(VAR_PHYS_INPUT,VarNumber,(DatasRead & Mask)?1:0);
				Mask = Mask<<1;
			}
		}
	}


}

void WritePhysicalOutputs( )
{
	int NumConf;
	StrIOConf * pConf;
	unsigned int DatasToWrite;
	unsigned int Mask;
	int VarNumber;
	for( NumConf=0; NumConf<NBR_OUTPUTS_CONF; NumConf++ )
	{
		pConf = &InfosGene->OutputsConf[ NumConf ];
		/* something mapped ? */
		if ( pConf->FirstClassicLadderIO!=-1 )
		{
			DatasToWrite = 0;
			/* get the datas to write from outputs variables */
			Mask = 1<<pConf->FirstChannel;
			for (VarNumber=pConf->FirstClassicLadderIO; VarNumber<pConf->FirstClassicLadderIO+pConf->NbrConsecutivesChannels; VarNumber++)
			{
				if ( ReadVar(VAR_PHYS_OUTPUT,VarNumber) )
					DatasToWrite = DatasToWrite | Mask;
				Mask = Mask<<1;
			}
			if ( pConf->FlagInverted )
				DatasToWrite = ~DatasToWrite;
			/* write the physical outputs */
#ifdef MAKE_IO_ACCESS
			if ( pConf->DeviceType==DEVICE_TYPE_DIRECT_ACCESS )
			{
				outb( DatasToWrite, pConf->SubDevOrAdr );
			}
#endif
#ifdef COMEDI_SUPPORT
			if ( pConf->DeviceType>=DEVICE_TYPE_COMEDI )
			{
				int ComediNum = pConf->DeviceType-DEVICE_TYPE_COMEDI;
				/* device is opened ? */
				if ( ComediDev[ ComediNum ] )
				{
					/* try to use the optimized way if possible */
					if ( pConf->FirstChannel<=31 && pConf->NbrConsecutivesChannels+pConf->FirstChannel<=32 )
					{
						int WriteMask = (1<<pConf->NbrConsecutivesChannels) - 1;
						WriteMask = WriteMask<<pConf->FirstChannel;
						comedi_dio_bitfield( ComediDev[ ComediNum ],
									pConf->SubDevOrAdr, WriteMask, &DatasToWrite );
					}
					else
					{
						int ScanEach;
						Mask = 1<<pConf->FirstChannel;
						for( ScanEach=pConf->FirstChannel; ScanEach<pConf->FirstChannel+pConf->NbrConsecutivesChannels; ScanEach++ )
						{
							int BitToWrite = DatasToWrite&Mask;
							comedi_dio_write( ComediDev[ ComediNum ],
									pConf->SubDevOrAdr, ScanEach, BitToWrite?1:0 );
							Mask = Mask<<1;
						}
					}
				}
			}
#endif
		}
	}

}

void InitHardware( void )
{
}

/* for Comedi under RTLinux, is called from init_module( ) and the mask parameter given tell
   which comedi devices to open.
   Under Linux the mask parameter is not used and is taken from the I/O file previously loaded */
void OpenHardware( int ComediToOpenMask )
{
#ifdef COMEDI_SUPPORT
	int ComediNum;
	int Mask;
#ifndef __RTL__
	int Pass;
	HardwareErrMsgToDisplay[ 0 ] = '\0';
	/* determine the comedi devices to open from the I/O Mapping Config */
	ComediToOpenMask = 0;
	for( Pass=0; Pass<2; Pass++)
	{
		StrIOConf * pConf;
		int NumConf;
		int NbrConf = (Pass==0)?NBR_INPUTS_CONF:NBR_OUTPUTS_CONF;
		for( NumConf=0; NumConf<NbrConf; NumConf++ )
		{
			pConf = (Pass==0)?&InfosGene->InputsConf[ NumConf ]:&InfosGene->OutputsConf[ NumConf ];
			if ( pConf->FirstClassicLadderIO!=-1 )
			{
				if ( pConf->DeviceType>=DEVICE_TYPE_COMEDI )
					ComediToOpenMask = ComediToOpenMask | (1<<(pConf->DeviceType-DEVICE_TYPE_COMEDI));
			}
		}
	}
#endif

	debug_printf("Open the Comedi hardware...\n");
	/* Open each comedi devices */
	Mask = 0x01;
	for( ComediNum=0; ComediNum<NBR_COMEDI_DEVICES; ComediNum++ )
	{
		ComediDev[ ComediNum ] = 0; /* not opened */
		if ( ComediToOpenMask & Mask )
		{
			char Buffer[ 20 ];
			sprintf( Buffer, "/dev/comedi%d", ComediNum );
			debug_printf( "Opening %s...", Buffer );
			ComediDev[ ComediNum ] = comedi_open( Buffer );
			if ( !ComediDev[ ComediNum ] )
			{
				debug_printf("failed!!!\n");
				sprintf( InfosGene->HardwareErrMsgToDisplay, "Failed to open /dev/comedi%d", ComediNum );
			}
			else
			{
				debug_printf("Ok!\n");
			}
		}
		Mask = Mask<<1;
	}
#endif
}

/* Under Linux, call ioperm( ) for the necessary adresses ports to access. */
/* For Comedi, configure the direction of bidirectional lines. */
void ConfigHardware( void )
{
	int NumConf;
	int NbrConf;
	int Pass;
	StrIOConf * pConf;
	// stop the logic refresh before configuring the following...
	StopRunIfRunning( );
#ifdef MODULE
	debug_printf("Configure the hardware...\n");
#else
	printf("Configure the hardware...\n");
#endif
	for( Pass=0; Pass<2; Pass++)
	{
		NbrConf = (Pass==0)?NBR_INPUTS_CONF:NBR_OUTPUTS_CONF;
		for( NumConf=0; NumConf<NbrConf; NumConf++ )
		{
			pConf = (Pass==0)?&InfosGene->InputsConf[ NumConf ]:&InfosGene->OutputsConf[ NumConf ];
			if ( pConf->FirstClassicLadderIO!=-1 )
			{
#if !defined( MODULE ) && defined( MAKE_IO_ACCESS )
				if ( pConf->DeviceType==DEVICE_TYPE_DIRECT_ACCESS )
				{
					if ( ioperm( pConf->SubDevOrAdr, 1, 1 ) )
					{
						printf("!!!Failed in ioperm call for the I/O port : 0x%X (you must be root)\n", pConf->SubDevOrAdr );
						pConf->FirstClassicLadderIO = -1;
						printf("!!!This direct access port has been disabled in the I/O conf to avoid to crash later...\n" );
						sprintf( InfosGene->HardwareErrMsgToDisplay, "Failed in ioperm call for the I/O port : 0x%X (you must be root)", pConf->SubDevOrAdr );
					}
				}
#endif
#ifdef COMEDI_SUPPORT
				if ( pConf->DeviceType>=DEVICE_TYPE_COMEDI )
				{
					int ScanEach;
					int ComediNum = pConf->DeviceType-DEVICE_TYPE_COMEDI;
					/* device is opened ? */
					if ( ComediDev[ ComediNum ] )
					{
						for( ScanEach=pConf->FirstChannel; ScanEach<pConf->FirstChannel+pConf->NbrConsecutivesChannels; ScanEach++ )
						{
							comedi_dio_config( ComediDev[ ComediNum ],
									pConf->SubDevOrAdr, ScanEach, (Pass==0)?COMEDI_INPUT:COMEDI_OUTPUT );
						}
					}
				}
#endif
			}
		}
	}
	// start the logic now !
	RunBackIfStopped( );
}

