/* Classic Ladder Project */
/* Copyright (C) 2001-2010 Marc Le Douarain */
/* http://www.multimania.com/mavati/classicladder */
/* http://www.sourceforge.net/projects/classicladder */
/* May 2010 */
/* -------------------------------------- */
/* Serial low-level functions for Windows */
/* -------------------------------------- */

/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "serial_common.h"
#include "protocol_modbus_master.h"
extern StrModbusConfig ModbusConfig;
void DoPauseMilliSecs( int MilliSecsTime );

char PortIsOpened = FALSE;
HANDLE hSerialComm = INVALID_HANDLE_VALUE;
int ResponseSizeExpected = 0;

char SerialOpen( )
{
	char tcNomPort[ 32 ];
	char tcInitPort[ 256 ];
	DCB dcb;

	/* if port already opened => close it before */
	if ( PortIsOpened )
		SerialClose( );

	sprintf(tcNomPort, "\\\\.\\%s",ModbusConfig.ModbusSerialPortNameUsed);
	hSerialComm = CreateFile( tcNomPort,
									GENERIC_READ | GENERIC_WRITE,
									0, 0, OPEN_EXISTING, 0, 0);
	if ( hSerialComm!=INVALID_HANDLE_VALUE )
	{
		FillMemory(&dcb, sizeof(dcb), 0);
		dcb.DCBlength = sizeof( dcb );
//TODO: see parity/bits/stops parameters...
		sprintf(tcInitPort,"%d,%c,%d,1",ModbusConfig.ModbusSerialSpeed,'n',8);
		if (BuildCommDCB( tcInitPort , &dcb))
		{
			dcb.fDtrControl = DTR_CONTROL_ENABLE;
			if ( !ModbusConfig.ModbusSerialUseRtsToSend )
				dcb.fRtsControl = RTS_CONTROL_ENABLE;
			if (SetCommState( hSerialComm, &dcb ))
			{
				PortIsOpened = TRUE;
			}
		}
	}
	if ( !PortIsOpened )
		printf( "Failed to open serial port %s !!!\n", ModbusConfig.ModbusSerialPortNameUsed );

	return PortIsOpened;
}

void SerialClose( )
{
	if ( PortIsOpened )
	{
		PortIsOpened = FALSE;
		CloseHandle( hSerialComm);
	}
}

char SerialPortIsOpened( void )
{
	return PortIsOpened;
}

void SerialSetRTS( int State )
{
	if ( PortIsOpened )
	{
		if ( ModbusConfig.ModbusDebugLevel>=3 )
			printf( "Set RTS=%d\n", State );
		EscapeCommFunction(hSerialComm, State?SETRTS:CLRRTS);
	}
}

void SerialSend( char *Buff, int BuffLength )
{
	if ( PortIsOpened )
	{
		unsigned long ulNbr;
		if ( ModbusConfig.ModbusSerialUseRtsToSend )
		{
			SerialSetRTS( 1 );
		}
		if ( ModbusConfig.ModbusDebugLevel>=2 )
			printf("Serial writing...\n");
		WriteFile( hSerialComm, Buff, BuffLength, &ulNbr, NULL);
		if ( ModbusConfig.ModbusDebugLevel>=2 )
			printf("Writing done!\n");
		if ( ModbusConfig.ModbusSerialUseRtsToSend )
		{
			// wait until everything has been transmitted
			FlushFileBuffers( hSerialComm );
//premiers essais avec mon module AVR...
DoPauseMilliSecs( 10 );
			SerialSetRTS( 0 );
		}
	}
}

void SerialSetResponseSize( int Size, int TimeOutResp )
{
	if ( PortIsOpened )
	{
		COMMTIMEOUTS timeouts;
		if ( ModbusConfig.ModbusDebugLevel>=2 )
			printf("Serial config...(SizeExpected=%d;TimeOut=%d\n",Size,TimeOutResp);
		/* 
ReadIntervalTimeout:
    Specifies the maximum acceptable time, in milliseconds, to elapse between the arrival of two characters on the communication line.
    In Windows Embedded CE, during a ReadFile operation, the time period begins immediately.
    If the interval between the arrivals of two characters exceeds the time amount specified in ReadIntervalTimeout, the ReadFile operation is completed and buffered data is returned.
A value of zero indicates that interval timeouts are not used.

ReadTotalTimeoutMultiplier:
    Specifies the multiplier, in milliseconds, used to calculate the total timeout period for read operations.
    For each read operation, this value is multiplied by the requested number of bytes to be read.
ReadTotalTimeoutConstant:
    Specifies the constant, in milliseconds, used to calculate the total timeout period for read operations.
    For each read operation, this value is added to the product of the ReadTotalTimeoutMultiplier member and the requested number of bytes.
    A value of zero for the ReadTotalTimeoutMultiplier and ReadTotalTimeoutConstant members indicates that total timeouts are not used for read operations.
*/
		timeouts.ReadIntervalTimeout = 20;
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = TimeOutResp;
		timeouts.WriteTotalTimeoutMultiplier = 2;
		timeouts.WriteTotalTimeoutConstant = 60;
		SetCommTimeouts( hSerialComm, &timeouts );
		ResponseSizeExpected = Size;
	}
}

int SerialReceive( char * Buff, int MaxBuffLength )//, int TimeOutResp )
{
	DWORD NbrCarsReceived = 0;
	if ( PortIsOpened )
	{
		if ( ReadFile( hSerialComm, Buff, MaxBuffLength/*ResponseSizeExpected*/, &NbrCarsReceived, NULL )!=TRUE )
			return 0;
	}
	return NbrCarsReceived;
}

void SerialPurge( void )
{
	if ( PortIsOpened )
	{
		if ( ModbusConfig.ModbusDebugLevel>=2 )
			printf("Serial flush all!\n");
		PurgeComm( hSerialComm, PURGE_RXCLEAR );
	}
}

